@@ -58,7 +58,7 @@ struct _Outstanding
   DConfEngineMessage dcem;
   volatile guint32 serial;
 
-  gchar *reset_path, *set_key;
+  gchar *set_key;
   GVariant *set_value;
 
   GTree *tree;
@@ -74,18 +74,10 @@ dconf_settings_backend_new_outstanding (DConfSettingsBackend *dcsb,
   Outstanding *outstanding;
 
   outstanding = g_slice_new (Outstanding);
-  outstanding->reset_path = NULL;
   outstanding->set_key = NULL;
   outstanding->dcem = *dcem;
 
-  if (!set_key || g_str_has_suffix (set_key, "/"))
-    {
-      g_assert (set_value == NULL);
-      outstanding->reset_path = g_strdup (set_key);
-    }
-  else
-    outstanding->set_key = g_strdup (set_key);
-
+  outstanding->set_key = g_strdup (set_key);
   outstanding->serial = 0;
 
   if (set_value)
@@ -145,7 +137,6 @@ dconf_settings_backend_remove_outstanding (DConfSettingsBackend  *dcsb,
 
         g_static_mutex_unlock (&dcsb->lock);
 
-        g_free (tmp->reset_path);
         g_free (tmp->set_key);
 
         if (tmp->set_value)
@@ -215,17 +206,7 @@ dconf_settings_backend_scan_outstanding (DConfSettingsBackend  *backend,
 
   for (node = backend->outstanding; node; node = node->next)
     {
-      if (node->reset_path)
-        {
-          if (g_str_has_prefix (key, node->reset_path))
-            {
-              *value = NULL;
-              found = TRUE;
-              break;
-            }
-        }
-
-      else if (node->set_key)
+      if (node->set_key)
         {
           if (strcmp (key, node->set_key) == 0)
             {
@@ -244,7 +225,7 @@ dconf_settings_backend_scan_outstanding (DConfSettingsBackend  *backend,
           gpointer result;
 
           if (dconf_settings_backend_scan_outstanding_tree (node->tree, key,
-                                                    length, &result))
+                                                            length, &result))
             {
               if (result)
                 *value = g_variant_ref (result);
@@ -295,7 +276,7 @@ dconf_settings_backend_incoming_signal (DConfSettingsBackend  *dcsb,
     }
 }
 
-static gboolean
+static GDBusMessage *
 dconf_settings_backend_filter (GDBusConnection *connection,
                                GDBusMessage    *message,
                                gboolean         is_incoming,
@@ -306,7 +287,7 @@ dconf_settings_backend_filter (GDBusConnection *connection,
   gchar **ae;
 
   if (!is_incoming)
-    return FALSE;
+    return message;
 
   if (connection == dcsb->session_bus)
     {
@@ -325,14 +306,21 @@ dconf_settings_backend_filter (GDBusConnection *connection,
   switch (g_dbus_message_get_message_type (message))
     {
     case G_DBUS_MESSAGE_TYPE_METHOD_RETURN:
-      return dconf_settings_backend_remove_outstanding (dcsb, bus_type,
-                                                        message, ae);
+      if (dconf_settings_backend_remove_outstanding (dcsb, bus_type,
+                                                     message, ae))
+        {
+          /* consume message */
+          g_object_unref (message);
+          return NULL;
+        }
+      else
+        return message;
 
     case G_DBUS_MESSAGE_TYPE_SIGNAL:
       dconf_settings_backend_incoming_signal (dcsb, bus_type, message, ae);
 
     default:
-      return FALSE;
+      return message;
     }
 }
 
@@ -343,7 +331,6 @@ dconf_settings_backend_read (GSettingsBackend   *backend,
                              gboolean            default_value)
 {
   DConfSettingsBackend *dcsb = (DConfSettingsBackend *) backend;
-  DConfReadType type;
 
   if (!default_value)
     {
@@ -352,26 +339,10 @@ dconf_settings_backend_read (GSettingsBackend   *backend,
       if (dconf_settings_backend_scan_outstanding (dcsb, key, &value))
         return value;
 
-      type = DCONF_READ_NORMAL;
+      return dconf_engine_read (dcsb->engine, key);
     }
   else
-    type = DCONF_READ_RESET;
-
-  return dconf_engine_read (dcsb->engine, key, type);
-}
-
-static gchar **
-dconf_settings_backend_list (GSettingsBackend  *backend,
-                             const gchar       *dir,
-                             gchar            **resets,
-                             gsize              n_resets,
-                             gsize             *length)
-{
-  DConfSettingsBackend *dcsb = (DConfSettingsBackend *) backend;
-
-  g_assert (n_resets == 0);
-
-  return dconf_engine_list (dcsb->engine, dir, NULL, length);
+    return dconf_engine_read_default (dcsb->engine, key);
 }
 
 static void
@@ -380,18 +351,21 @@ dconf_settings_backend_send (GDBusConnection    *bus,
                              volatile guint32   *serial)
 {
   GDBusMessage *message;
+  GVariant *body;
 
   message = g_dbus_message_new_method_call (dcem->destination,
                                             dcem->object_path,
                                             dcem->interface,
                                             dcem->method);
-  g_dbus_message_set_body (message, dcem->body);
+  body = g_variant_get_child_value (dcem->body, 0);
+  g_dbus_message_set_body (message, body);
+  g_variant_unref (body);
 
   if (serial)
-    g_dbus_connection_send_message (bus, message, serial, NULL);
+    g_dbus_connection_send_message (bus, message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, serial, NULL);
   else
-    g_dbus_connection_send_message_with_reply_sync (bus, message, -1,
-                                                    NULL, NULL, NULL);
+    g_dbus_connection_send_message_with_reply_sync (bus, message, G_DBUS_SEND_MESSAGE_FLAGS_NONE,
+                                                    -1, NULL, NULL, NULL);
 
   g_variant_unref (dcem->body);
   g_object_unref (message);
@@ -453,7 +427,7 @@ dconf_settings_backend_get_bus (DConfSettingsBackend  *dcsb,
 
 static gboolean
 dconf_settings_backend_write (GSettingsBackend *backend,
-                              const gchar      *path_or_key,
+                              const gchar      *key,
                               GVariant         *value,
                               gpointer          origin_tag)
 {
@@ -462,24 +436,17 @@ dconf_settings_backend_write (GSettingsBackend *backend,
   DConfEngineMessage dcem;
   GDBusConnection *bus;
 
-  if (!dconf_engine_write (dcsb->engine, &dcem, path_or_key, value, NULL))
+  if (!dconf_engine_write (dcsb->engine, key, value, &dcem, NULL))
     return FALSE;
 
   if (!dconf_settings_backend_get_bus (dcsb, &bus, &dcem))
     return FALSE;
 
-  serial = dconf_settings_backend_new_outstanding (dcsb,
-                                                   &dcem,
-                                                   path_or_key,
-                                                   value,
-                                                   NULL);
-
+  serial = dconf_settings_backend_new_outstanding (dcsb, &dcem, key,
+                                                   value, NULL);
   dconf_settings_backend_send (bus, &dcem, serial);
 
-  if (g_str_has_suffix (path_or_key, "/"))
-    g_settings_backend_path_changed (backend, path_or_key, origin_tag);
-  else
-    g_settings_backend_changed (backend, path_or_key, origin_tag);
+  g_settings_backend_changed (backend, key, origin_tag);
 
   return TRUE;
 }
@@ -499,8 +466,8 @@ dconf_settings_backend_write_tree (GSettingsBackend *backend,
 
   g_settings_backend_flatten_tree (tree, &prefix, &keys, &values);
 
-  if (dconf_engine_write_many (dcsb->engine, &dcem,
-                               prefix, keys, values, NULL))
+  if (dconf_engine_write_many (dcsb->engine,
+                               prefix, keys, values, &dcem, NULL))
     {
       if (dconf_settings_backend_get_bus (dcsb, &bus, &dcem))
         {
@@ -524,10 +491,10 @@ dconf_settings_backend_write_tree (GSettingsBackend *backend,
 
 static void
 dconf_settings_backend_reset (GSettingsBackend *backend,
-                              const gchar      *path_or_key,
+                              const gchar      *key,
                               gpointer          origin_tag)
 {
-  g_assert_not_reached ();
+  dconf_settings_backend_write (backend, key, NULL, origin_tag);
 }
 
 static gboolean
@@ -535,13 +502,8 @@ dconf_settings_backend_get_writable (GSettingsBackend *backend,
                                      const gchar      *name)
 {
   DConfSettingsBackend *dcsb = (DConfSettingsBackend *) backend;
-  DConfEngineMessage dcem;
-  GDBusConnection *bus;
 
-  if (!dconf_engine_is_writable (dcsb->engine, &dcem, name, NULL))
-    return FALSE;
-
-  return dconf_settings_backend_get_bus (dcsb, &bus, &dcem);
+  return dconf_engine_is_writable (dcsb->engine, name);
 }
 
 static void
@@ -552,7 +514,7 @@ dconf_settings_backend_subscribe (GSettingsBackend *backend,
   DConfEngineMessage dcem;
   GDBusConnection *bus;
 
-  dconf_engine_watch (dcsb->engine, &dcem, name);
+  dconf_engine_watch (dcsb->engine, name, &dcem);
 
   if (dconf_settings_backend_get_bus (dcsb, &bus, &dcem))
     dconf_settings_backend_send (bus, &dcem, NULL);
@@ -566,7 +528,7 @@ dconf_settings_backend_unsubscribe (GSettingsBackend *backend,
   DConfEngineMessage dcem;
   GDBusConnection *bus;
 
-  dconf_engine_unwatch (dcsb->engine, &dcem, name);
+  dconf_engine_unwatch (dcsb->engine, name, &dcem);
 
   if (dconf_settings_backend_get_bus (dcsb, &bus, &dcem))
     dconf_settings_backend_send (bus, &dcem, NULL);
@@ -594,9 +556,23 @@ dconf_settings_backend_sync (GSettingsBackend *backend)
   g_static_mutex_unlock (&dcsb->lock);
 }
 
+static GVariant *
+dconf_settings_backend_service_func (DConfEngineMessage *dcem)
+{
+  g_assert (dcem->bus_type == 'e');
+
+  return g_dbus_connection_call_sync (g_bus_get_sync (G_BUS_TYPE_SESSION,
+                                                      NULL, NULL),
+                                      dcem->destination, dcem->object_path,
+                                      dcem->interface, dcem->method,
+                                      dcem->body, dcem->reply_type,
+                                      0, -1, NULL, NULL);
+}
+
 static void
 dconf_settings_backend_init (DConfSettingsBackend *dcsb)
 {
+  dconf_engine_set_service_func (dconf_settings_backend_service_func);
   dcsb->engine = dconf_engine_new (NULL);
 }
 
@@ -604,11 +580,9 @@ static void
 dconf_settings_backend_class_init (GSettingsBackendClass *class)
 {
   class->read = dconf_settings_backend_read;
-  class->list = dconf_settings_backend_list;
   class->write = dconf_settings_backend_write;
-  class->write_keys = dconf_settings_backend_write_tree;
+  class->write_tree = dconf_settings_backend_write_tree;
   class->reset = dconf_settings_backend_reset;
-  class->reset_path = dconf_settings_backend_reset;
   class->get_writable = dconf_settings_backend_get_writable;
   class->subscribe = dconf_settings_backend_subscribe;
   class->unsubscribe = dconf_settings_backend_unsubscribe;
